import os 
import json
import codecs

__different_files = {}

def ordered_json(obj):
    if isinstance(obj, dict):
        return sorted((k, ordered_json(v)) for k, v in obj.items())
    if isinstance(obj, list):
        return sorted(ordered_json(x) for x in obj)
    else:
        return obj

def __compare(pathA: str, pathB: str) -> None:
    if os.path.isfile(pathA) and os.path.isfile(pathB): # file name of the 2 files should be same
        if not pathA.endswith('.json') and not pathA.endswith('.jsonc'):
            return

        print(pathA)
        # I dont want to handle comment in json
        with codecs.open(pathA, 'r', 'utf-8') as f:
            objA = f.read()
        
        with codecs.open(pathB, 'r', 'utf-8') as f:
            objB = f.read()

        if objA != objB:
            __different_files[pathA] = 'different'

    elif os.path.isdir(pathA) and os.path.isdir(pathB):
        fA = set(os.listdir(pathA))
        fB = set(os.listdir(pathB))

        for fa in fA:
            nextA = os.path.join(pathA, fa)
            if fa not in fB:
                __different_files[nextA] = 'not found in B'
            else: # same file/dir name
                nextB = os.path.join(pathB, fa)
                __compare(nextA, nextB)

        for fb in fB:
            nextB = os.path.join(pathB, fb)
            if fb not in fA:
                __different_files[nextB] = 'not found in A'
        pass
    else:
        __different_files[pathA] = f"file-dir-mismatch:{pathB}"
    return

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-A', type=str, default='.', help='path to source repository (your rundown)')
    parser.add_argument('-B', type=str, required=True, help='path to the other repository that (is supposed to) holds identical directory structures (the other rundown)')
    args = parser.parse_args()

    if not os.path.exists(args.A) or not os.path.isdir(args.A):
        raise ValueError(f'-A {args.A} is not a directory')
    if not os.path.exists(args.B) or not os.path.isdir(args.B):
        raise ValueError(f'-B {args.B} is not a directory')

    print('Performing check...')
    __compare(args.A, args.B)
    with open('Compare.json', 'w') as f:
        f.write(json.dumps(__different_files, indent=4))
        